﻿using System.Security.Claims;
using Infrastructure.Utils;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.IdentityModel.JsonWebTokens;

namespace Infrastructure.Security;

public interface ITokenBuilder
{
    IList<Claim> GetClaimsByUserContext(IUserContext userContext);

    void SetUserContext(TokenValidatedContext context);
}

public class TokenBuilder : ITokenBuilder
{
    public IList<Claim> GetClaimsByUserContext(IUserContext userContext)
    {
        var claims = new List<Claim>()
        {
            new(JwtRegisteredClaimNames.UniqueName, userContext.Username),
            new(JwtRegisteredClaimNames.NameId, userContext.Id.ToString()),
            new(JwtRegisteredClaimNames.Name, userContext.Name),
            new(JwtRegisteredClaimNames.Email, userContext.Email),
        };
        claims.AddRange(userContext.RoleIds.Select(rId => new Claim(ClaimTypes.Role, rId)));
        return claims;
    }

    public void SetUserContext(TokenValidatedContext context)
    {
        var userContext = context.HttpContext.RequestServices.GetService<IUserContext>() ??
                          throw new NullReferenceException(nameof(IUserContext));
        var principal = context.Principal ?? throw new NullReferenceException(nameof(context.Principal));
        userContext.Id = long.Parse(
            principal.Claims.FirstOrDefault(c => c.Type == JwtRegisteredClaimNames.NameId)?.Value);
        userContext.Username =
            principal.Claims.FirstOrDefault(c => c.Type == JwtRegisteredClaimNames.UniqueName)?.Value!;
        userContext.Name = principal.Claims.FirstOrDefault(c => c.Type == JwtRegisteredClaimNames.Name)?.Value!;
        userContext.Email = principal.Claims.FirstOrDefault(c => c.Type == JwtRegisteredClaimNames.Email)?.Value!;
        userContext.RoleIds = principal.Claims.Where(c => c.Type == ClaimTypes.Role).Select(c => c.Value).ToArray();
        userContext.RemoteIpAddress = context.HttpContext.GetRequestIp();
    }
}